#!/usr/bin/env python
# coding: utf-8
# Simple interactive demo
import sys

from ytrans import YTranslator, read_key

token_trim = lambda token: token.rstrip(" ").lstrip(" ").strip("\n")


def do_show_menu(aux_message=''):
    sys.stdout.write("""
        %s
        c   -- Change the language
        d   -- Detect language
        m   -- Show this menu
        p   -- Get available primary languages
        r   -- Get available translations for a language code
        t   -- Translate content
        q   -- Exit the program
    \n""" % aux_message)


def do_lang_listing(ytrans):
    sys.stdout.write("\n".join(ytrans.get_supported_primaries()))


def do_supported_translations_from_lang(ytrans, lang):
    text = prompt(
        pre_text="Primary language, leave blank to use '%s" % lang).strip('\n')

    if text:
        lang = text

    results = ytrans.get_supported_translations(lang)
    if not results:
        msg = "No translations supported for %s" % lang
    else:
        msg = "\n".join(results)
    sys.stdout.write(msg)


def do_detect_lang(ytrans):
    text = prompt(pre_text='Enter text to detect')
    sys.stdout.write(ytrans.detect(text.strip('\n')))


def set_lang(ytrans, current_lang):
    lang_in = prompt(
        "\n[Current: %s. Enter language code eg ru:$> " % current_lang)
    if lang_in:
        current_lang = token_trim(lang_in)

    if not ytrans.is_valid_lang(current_lang):
        sys.stdout.write("%s is an invalid language!" % current_lang)
        return None

    return current_lang


def do_translate(ytrans, lang, lines):
    tokens = [token_trim(token) for token in lines if token]
    sys.stdout.write("Performing translation")
    translated = ytrans.translate(lang=lang, text_collection=tokens)
    if translated and tokens:
        sys.stdout.write("\n")
        for i, equiv in enumerate(translated):
            sys.stdout.write("%s <=> %s\n" % (tokens[i], equiv))
            sys.stdout.write("\n")


def do_translation(ytrans, dest_lang):
    sys.stdout.write("* Hit Ctrl-D to process *\nText separated by 'Enter'\n$> ")
    do_translate(ytrans, dest_lang, sys.stdin.read().split('\n'))


def do_report_invalid_opt(opt_str):
    sys.stderr.write("Invalid option: '%s'\n" % opt_str)


def do_exit():
    raise KeyboardInterrupt


def prompt(ps='$>', pre_text=''):
    sys.stdout.write('%s %s ' % (pre_text, ps)) and sys.stdout.flush()
    return sys.stdin.readline()


class Console(object):
    def __init__(self, ps='$>', stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
        self._stdin = stdin
        self._stdout = stdout
        self._stderr = stderr
        self._ps = ps

    def write(self, *args, **kwargs):
        self._stdout.write(*args, **kwargs) and self._stdout.flush()


class CommandExecutor(object):
    """
    Register of commands
    """
    def __init__(self):
        self._actions = {}
        self._dependences = {}

    def add_command(self, name, action, *dependence):
        # if name is None register default command
        self._actions[name] = (action, dependence)

    def remove_command(self, name):
        assert name in self._actions,\
            u"Command {} not found!".format(name)
        del self._actions[name]

    def set_dependences(self, **kwargs):
        self._dependences.update(kwargs)

    def process_command(self, command):
        items = self._actions.get(command)
        if items is None:
            items = self._actions.get(None)
        assert items is not None, "Invalid command!"

        action, dependence = items
        return action(*self._get_args(dependence))

    def _get_args(self, dependence):
        for d in dependence:
            assert d in self._dependences,\
                u"Dependence {} not found!".format(d)
            yield self._dependences[d]


command_executor = CommandExecutor()
command_executor.add_command('d', do_detect_lang, 'translator')
command_executor.add_command('p', do_lang_listing, 'translator')
command_executor.add_command('r', do_supported_translations_from_lang, 'translator', 'lang')
command_executor.add_command('t', do_translation, 'translator', 'lang')
command_executor.add_command('m', do_show_menu, 'aux_message')
command_executor.add_command('q', do_exit)
command_executor.add_command(None, do_report_invalid_opt, 'opt_str')


def main():
    api_key = read_key()
    ytrans = YTranslator(api_key)
    lang = 'ru'
    command_executor.set_dependences(translator=ytrans, lang=lang)
    aux_message = "* Hit Ctrl-C to exit *. Target Language: '%s'"
    do_show_menu(aux_message % lang)

    while True:
        try:
            opt_str = prompt(pre_text='Enter your option').strip('\n') or '^'
            l_opt_ch = opt_str.lower()[0]
            if l_opt_ch == 'c':
                tmp = set_lang(ytrans, lang)
                if tmp:
                    lang = tmp
                    sys.stdout.write("Language set to '%s'\n" % lang)
            else:
                command_executor.set_dependences(
                    lang=lang, opt_str=opt_str, aux_message=aux_message % lang)
                command_executor.process_command(l_opt_ch)

        except KeyboardInterrupt:
            break
        except Exception as e:
            sys.stderr.write('%s\n' % e)
            sys.stderr.flush()

    sys.stdout.write("\nBye\n")


if __name__ == "__main__":
    main()
